// ProgressData.cpp: implementation of the ProgressData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "rootfly.h"
#include "ProgressData.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ProgressData::ProgressData()
{
  Reset();
}

ProgressData::~ProgressData()
{

}

void ProgressData::SetExists(const int &tube, const int &window, const int &session)
{
  if ((tube < 0) || (window < 0) || (session < 0))
    return;

  // ensure exist_list is large enough
  if (m_exist_list.size() <= tube)
  {
    m_exist_list.resize(tube + 1, std::vector< std::vector<bool> >());
  }
  if (m_exist_list[tube].size() <= window)
  {
    m_exist_list[tube].resize(window + 1, std::vector<bool>());
  }
  if (m_exist_list[tube][window].size() <= session)
  {
    m_exist_list[tube][window].resize(session + 1, false);
  }

  // update list and counter
  if (!m_exist_list[tube][window][session])
  {
    m_exist_count++;
  }
  m_exist_list[tube][window][session] = true;

  
  if (m_tube_list.size() <= tube)
  {
    m_tube_list.resize(tube + 1, false);
  }
  if (m_window_list.size() <= window)
  {
    m_window_list.resize(window + 1, false);
  }
  if (m_session_list.size() <= session)
  {
    m_session_list.resize(session + 1, false);
  }

  if (!m_tube_list[tube])
  {
    m_tube_count++;
    m_tube_list[tube] = true;
  }
  if (!m_window_list[window])
  {
    m_window_count++;
    m_window_list[window] = true;
  }
  if (!m_session_list[session])
  {
    m_session_count++;
    m_session_list[session] = true;
  }
}

bool ProgressData::GetExists(const int &tube, const int &window, const int &session)
{
  if ((tube < 0) || (window < 0) || (session < 0))
  {
    return false;
  }

  if ((m_exist_list.size() <= tube) || 
    (m_exist_list[tube].size() <= window) ||
    (m_exist_list[tube][window].size() <= session))
  {
    return false;
  }

  return m_exist_list[tube][window][session];
}

void ProgressData::SetAnnotated(const int &tube, const int &window, const int &session)
{
  if ((tube < 0) || (window < 0) || (session < 0))
    return;

  // ensure list is large enough
  if (m_annotated_list.size() <= tube)
  {
    m_annotated_list.resize(tube + 1, std::vector< std::vector<bool> >());
  }
  if (m_annotated_list[tube].size() <= window)
  {
    m_annotated_list[tube].resize(window + 1, std::vector<bool>());
  }
  if (m_annotated_list[tube][window].size() <= session)
  {
    m_annotated_list[tube][window].resize(session + 1, false);
  }

  // update list and counter
  if (!m_annotated_list[tube][window][session])
  {
    m_annotated_count++;
  }
  m_annotated_list[tube][window][session] = true;
}

// return true if the given TWS is marked as annotated
bool ProgressData::GetAnnotated(const int &tube, const int &window, const int &session)
{
  if ((tube < 0) || (window < 0) || (session < 0))
  {
    return false;
  }

  if ((m_annotated_list.size() <= tube) || 
    (m_annotated_list[tube].size() <= window) ||
    (m_annotated_list[tube][window].size() <= session))
  {
    return false;
  }

  return m_annotated_list[tube][window][session];
}

void ProgressData::SetDone(const int &tube, const int &window, const int &session)
{
  if ((tube < 0) || (window < 0) || (session < 0))
    return;

  // ensure list is large enough
  if (m_done_list.size() <= tube)
  {
    m_done_list.resize(tube + 1, std::vector< std::vector<bool> >());
  }
  if (m_done_list[tube].size() <= window)
  {
    m_done_list[tube].resize(window + 1, std::vector<bool>());
  }
  if (m_done_list[tube][window].size() <= session)
  {
    m_done_list[tube][window].resize(session + 1, false);
  }

  // update list and counter
  if (!m_done_list[tube][window][session])
  {
    m_done_count++;
  }
  m_done_list[tube][window][session] = true;
}

bool ProgressData::GetDone(const int &tube, const int &window, const int &session)
{
  if ((tube < 0) || (window < 0) || (session < 0))
    return false;

  if ((m_done_list.size() <= tube) || 
    (m_done_list[tube].size() <= window) ||
    (m_done_list[tube][window].size() <= session))
  {
    return false;
  }

  return m_done_list[tube][window][session];
}

const std::vector<bool>* ProgressData::GetTubeList()
{
  return &m_tube_list;
}


const std::vector<bool>* ProgressData::GetWindowList()
{
  return &m_window_list;
}


const std::vector<bool>* ProgressData::GetSessionList()
{
  return &m_session_list;
}

void ProgressData::Reset()
{
  m_exist_count = m_annotated_count = m_done_count = m_tube_count = m_window_count = m_session_count = 0;
  m_exist_list.clear();
  m_annotated_list.clear();
  m_done_list.clear();
  m_tube_list.clear();
  m_window_list.clear();
  m_session_list.clear();
}

void ProgressData::ResetAnnotated()
{
  m_annotated_count = 0;
  m_annotated_list.clear();
}

void ProgressData::ResetDone()
{
  m_done_count = 0;
  m_done_list.clear();
}

int ProgressData::GetImageCount()
{
  return m_exist_count;
}

int ProgressData::GetAnnotatedCount()
{
  return m_annotated_count;
}

int ProgressData::GetDoneCount()
{
  return m_done_count;
}

int ProgressData::GetTubeCount()
{
  return m_tube_count;
}

int ProgressData::GetWindowCount()
{
  return m_window_count;
}

int ProgressData::GetSessionCount()
{
  return m_session_count;
}